package com.codegeist.fisheye.internal.core.connection;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Status;

import com.codegeist.fisheye.core.Activator;
import com.codegeist.fisheye.core.connection.FisheyeConnection;
import com.codegeist.fisheye.core.connection.IConnectionListener;
import com.codegeist.fisheye.core.connection.IFisheyeConnectionManager;
import com.codegeist.fisheye.service.FisheyeServiceFactory;

public class FisheyeConnectionManager implements IFisheyeConnectionManager {

	private ListenerList listeners = new ListenerList();
	private List<FisheyeConnection> connections = new ArrayList<FisheyeConnection>();
	
	public void addConnectionListener(IConnectionListener listener) {
		synchronized (listeners) {
			listeners.add(listener);
		}
	}
	
	public void removeConnectionListener(IConnectionListener listener) {
		synchronized (listeners) {
			listeners.remove(listener);
		}
	}
	
	public FisheyeConnection createConnection(String name, String connectionUrl) {
		FisheyeConnection connection = new FisheyeConnection(name, connectionUrl, FisheyeServiceFactory.createService(connectionUrl));
		
		Object[] listenersCopy = listeners.getListeners();
		for (Object listener : listenersCopy) {
			try {
				((IConnectionListener)listener).connectionAdded(connection);
			} catch (Exception e) {
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Error creating connection", e));
			}
		}
		
		// TODO check for duplicate addition
		connections.add(connection);
		return connection;
	}

	public List<FisheyeConnection> getConnections() {
		return Collections.unmodifiableList(new ArrayList<FisheyeConnection>(connections));
	}
	
	public FisheyeConnection getConnectionByName(String connectionName) {
		if (connectionName == null) {
			throw new IllegalArgumentException("Connection name must be supplied");
		}
		
		for (FisheyeConnection connection : connections) {
			if (connectionName.equals(connection.getName())) {
				return connection;
			}
		}
		return null;
	}

	public void removeConnection(FisheyeConnection connection) {
		connections.remove(connection);
		
		Object[] listenersCopy = listeners.getListeners();
		for (Object listener : listenersCopy) {
			try {
				((IConnectionListener)listener).connectionRemoved(connection);
			} catch (Exception e) {
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Error removing connection", e));
			}
		}
	}
	
}
