package com.codegeist.fisheye.internal.service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.AssertionFailedError;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class MockHandler extends DefaultHandler {
	private static class Event {
		private boolean open;
		private String contents;
		
		public Event(boolean open, String contents) {
			this.open = open;
			setContents(contents);
		}
		
		public void setOpen(boolean open) {
			this.open = open;
		}
		
		public void setContents(String contents) {
			if (contents == null) {
				this.contents = "";
			} else {
				this.contents = contents.trim();
			}
		}
		
		@Override
		public boolean equals(Object obj) {
			Event that = (Event) obj;
			
			if (this.open != that.open) {
				return false;
			}
			
			return this.contents.equals(that.contents);
		}
	}
	
	private boolean gotStartDocument;
	private boolean gotEndDocument;
	
	private List<Event> expectations = new ArrayList<Event>();
	private List<Event> actuals = new ArrayList<Event>();
	private StringBuilder buffer = new StringBuilder();
	
	public void expectOpen(String name) {
		expectations.add(new Event(true, null));
	}
	
	public void expectElement(String name, String contents) {
		expectations.add(new Event(true, null));
		expectations.add(new Event(false, contents));
	}
	
	public void expectClose(String name) {
		expectations.add(new Event(false, null));
	}
	
	@Override
	public void startDocument() throws SAXException {
		gotStartDocument = true;
	}

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		buffer.setLength(0);
		actuals.add(new Event(true, localName));
	}
	
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		actuals.add(new Event(false, buffer.toString()));
		buffer.setLength(0);
	}
	
	@Override
	public void endDocument() throws SAXException {
		gotEndDocument = true;
	}
	
	public void assertExpectations() throws AssertionFailedError {
		if (expectations.size() != actuals.size()) {
			throw new AssertionFailedError("Expected " + expectations.size() + " events, got " + actuals.size());
		}
		
		if (!gotStartDocument) {
			throw new AssertionFailedError("Expected start document");
		}
		
		if (!gotEndDocument) {
			throw new AssertionFailedError("Expected end document");
		}		
		
		Iterator<Event> iActual = actuals.iterator();
		Iterator<Event> iExpected = expectations.iterator();
		
		while (iActual.hasNext()) {
			iActual.next().equals(iExpected.next());
		}
	}
}
