package com.codegeist.fisheye.internal.service;

import java.io.StringReader;

import org.junit.Before;
import org.junit.Test;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import com.codegeist.fisheye.service.FisheyeException;

import junit.framework.TestCase;

public class RestHandlerTest extends TestCase {

	private RestHandler handler;
	private XMLReader reader;
	private MockHandler mockHandler;

	@Before
	protected void setUp() throws Exception {
		reader = XMLReaderFactory.createXMLReader();
		mockHandler = new MockHandler();
		handler = new RestHandler(mockHandler);
		reader.setContentHandler(handler);
	}

	@Test
	public void testEmptyResponse() throws Exception {
		reader.parse(new InputSource(new StringReader("<response></response>")));
		mockHandler.assertExpectations();
	}
	
	@Test
	public void testGenericError() throws Exception {
		try {
			reader.parse(new InputSource(new StringReader("<error>Some unknown server error</error>")));
			fail("Expected exception");
		} catch (SAXException e) {
			Throwable cause = e.getException();
			assertNotNull(cause);
			assertTrue(cause instanceof FisheyeException);
			FisheyeException fisheyeException = (FisheyeException) cause;
			assertEquals("Some unknown server error", cause.getMessage());
			assertEquals(FisheyeException.ExceptionType.SERVER_ERROR, fisheyeException.getExceptionType());
		}
	}
	
	@Test
	public void testApiDisabledError() throws Exception {
		try {
			reader.parse(new InputSource(new StringReader("<error>API access is disabled</error>")));
			fail("Expected exception");
		} catch (SAXException e) {
			Throwable cause = e.getException();
			assertNotNull(cause);
			assertTrue(cause instanceof FisheyeException);
			FisheyeException fisheyeException = (FisheyeException) cause;
			assertEquals(FisheyeException.ExceptionType.INTERFACE_DISABLED, fisheyeException.getExceptionType());
		}
	}
	
	@Test
	public void testAuthenticationError() throws Exception {
		try {
			reader.parse(new InputSource(new StringReader("<error>authentication failed</error>")));
			fail("Expected exception");
		} catch (SAXException e) {
			Throwable cause = e.getException();
			assertNotNull(cause);
			assertTrue(cause instanceof FisheyeException);
			FisheyeException fisheyeException = (FisheyeException) cause;
			assertEquals(FisheyeException.ExceptionType.AUTHENTICATION_FAILED, fisheyeException.getExceptionType());
		}
	}
	
	@Test
	public void testResponseElement() throws Exception {
		reader.parse(new InputSource(new StringReader("<response><string>test</string></response>")));
		mockHandler.expectElement("string", "test");
		mockHandler.assertExpectations();
	}
}
