package com.codegeist.fisheye.internal.ui.search;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;

import com.codegeist.fisheye.core.connection.FisheyeConnection;
import com.codegeist.fisheye.domain.query.Query;
import com.codegeist.fisheye.service.FisheyeException;
import com.codegeist.fisheye.service.FisheyeService;
import com.codegeist.fisheye.ui.Activator;

public class FisheyeQuery implements ISearchQuery {

	private final Query query;
	private final FisheyeConnection connection;
	private FisheyeSearchResult fisheyeSearchResult;
	private final String repository;

	public FisheyeQuery(FisheyeConnection connection, String repository, Query query) {
		this.connection = connection;
		this.repository = repository;
		this.query = query;
	}
	
	public boolean canRerun() {
		return true;
	}

	public boolean canRunInBackground() {
		return true;
	}

	public String getLabel() {
		return connection.getName() + " - " + query.generateEyeQl();
	}

	public ISearchResult getSearchResult() {
		if (fisheyeSearchResult == null) {
			fisheyeSearchResult = new FisheyeSearchResult(this);
		}
		
		return fisheyeSearchResult;
	}

	public IStatus run(IProgressMonitor monitor) throws OperationCanceledException {
		FisheyeService service = connection.getService();
		try {
			service.query(null, repository, query, fisheyeSearchResult);
			return Status.OK_STATUS;
		} catch (FisheyeException e) {
			return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Error executing query", e);
		}
	}

}
